package polyglot.ext.op;

import polyglot.ext.jl.types.TypeSystem_c;
import polyglot.types.*;
import polyglot.util.*;
import java.util.*;

public class OPTypeSystem extends TypeSystem_c
{
    public PrimitiveType createPrimitive(PrimitiveType.Kind kind) {
        return new OPPrimitiveType_c(this, kind);
    }

    public ParsedClassType createClassType(LazyClassInitializer init) {
        return new OPParsedClassType_c(this, init);
    }

    private static final String WRAPPER_PACKAGE = "polyglot.ext.op.runtime";
    private static Map wrapper = null;
    private static Map getter = null;

    public MethodInstance getter(PrimitiveType t) {
        if (getter == null) {
            getter = new HashMap();
            getter.put(Boolean(), "booleanValue");
            getter.put(Char(), "charValue");
            getter.put(Short(), "shortValue");
            getter.put(Int(), "intValue");
            getter.put(Long(), "longValue");
            getter.put(Float(), "floatValue");
            getter.put(Double(), "doubleValue");
        }

        String methodName = (String) getter.get(t);
        ConstructorInstance ci = wrapper(t);

        for (Iterator i = ci.container().methods().iterator();
              i.hasNext(); ) {
            MethodInstance mi = (MethodInstance) i.next();
            if (mi.name().equals(methodName) && mi.formalTypes().isEmpty()) {
                return mi;
            }
        }

        throw new InternalCompilerError("Could not find getter for " + t);
    }

    public ConstructorInstance wrapper(PrimitiveType t) {
        if (wrapper == null) {
            wrapper = new HashMap();
            wrapper.put(Boolean(), "Boolean");
            wrapper.put(Char(), "Character");
            wrapper.put(Short(), "Short");
            wrapper.put(Int(), "Integer");
            wrapper.put(Long(), "Long");
            wrapper.put(Float(), "Float");
            wrapper.put(Double(), "Double");
        }

        String name = WRAPPER_PACKAGE + "." + (String) wrapper.get(t);

        try {
            Type ct = systemResolver().findType(name);

            for (Iterator i = ct.toClass().constructors().iterator();
                 i.hasNext(); ) {
                ConstructorInstance ci = (ConstructorInstance) i.next();
                if (ci.formalTypes().size() == 1) {
                    Type argType = (Type) ci.formalTypes().get(0);
                    if (isSame(argType, t)) {
                        return ci;
                    }
                }
            }
        }
        catch (SemanticException e) {
            throw new InternalCompilerError(e.getMessage());
        }

        throw new InternalCompilerError("Could not find constructor for " + t);
    }
}
