package polyglot.ext.polyj.visit;

import polyglot.ast.*;
import polyglot.frontend.*;
import polyglot.types.*;
import polyglot.visit.*;
import polyglot.ext.polyj.extension.*;

/**
 * This visitor is run by PolyJ immediately before translation.
 * It takes care of small tasks that need done,
 * rather than make each a separate pass.
 * 1. Number each static initializer block of a class, uniquely.
 */
public class PreTranslator extends HaltingVisitor {
    
    /**
     * Used to number uniquely each static initialization block in a class
     */ 
    protected int staticInit;
    
    public PreTranslator() {
        staticInit = 0;
    }
       
    public NodeVisitor enter(Node n) {
	    
	//System.out.println(n.toString()); 
	//System.out.println(n.del().getClass().getName());
	return ((PolyJExt)n.ext()).preTranslateEnter(this);
    }

    public Node leave(Node parent, Node old, Node n, NodeVisitor v) { 
	return ((PolyJExt)n.ext()).preTranslate((PreTranslator) v);
    }

    public int allocInitNum() {
        staticInit++;
        return staticInit-1;
    }
    
    public Object copy() {
        return super.copy();
    }
    
    public NodeVisitor enterClass(ClassDecl c) {
        PreTranslator tr = (PreTranslator) copy();
        tr.staticInit = 0;
        return tr;
    }
}
