/* Soot - a J*va Optimization Framework
 * Copyright (C) 2003 Ondrej Lhotak
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

package soot.jimple.paddle;
import soot.*;
import soot.jimple.paddle.queue.*;
import java.util.*;

/** Converts context-insensitive call edges into context-sensitive ones
 * for each reachable method context.
 * @author Ondrej Lhotak
 */
public class TradContextCallGraphBuilder extends AbsContextCallGraphBuilder 
{ 
    TradContextCallGraphBuilder( Rctxt_method methodsIn,
            Rsrcc_srcm_stmt_kind_tgtc_tgtm edgesIn,
            Qsrcc_srcm_stmt_kind_tgtc_tgtm out,
            AbsCallGraph cicg ) {
        super( methodsIn, edgesIn, out, cicg );
    }
    private MethodToContexts m2c = new MethodToContexts();
    public boolean update() {
        boolean change = false;
        for( Iterator eIt = edgesIn.iterator(); eIt.hasNext(); ) {
            final Rsrcc_srcm_stmt_kind_tgtc_tgtm.Tuple e = (Rsrcc_srcm_stmt_kind_tgtc_tgtm.Tuple) eIt.next();
            for( Iterator mcIt = m2c.get(e.srcm()).iterator(); mcIt.hasNext(); ) {
                final MethodOrMethodContext mc = (MethodOrMethodContext) mcIt.next();
                out.add( mc.context(), mc.method(), e.stmt(), e.kind(),
                        e.tgtc(), e.tgtm() );
                change = true;
            }
        }
        for( Iterator tIt = methodsIn.iterator(); tIt.hasNext(); ) {
            final Rctxt_method.Tuple t = (Rctxt_method.Tuple) tIt.next();
            Rsrcc_srcm_stmt_kind_tgtc_tgtm edges = cicg.edgesOutOf(t.method());
            Iterator it = edges.iterator();
            while( it.hasNext() ) {
                Rsrcc_srcm_stmt_kind_tgtc_tgtm.Tuple e =
                    (Rsrcc_srcm_stmt_kind_tgtc_tgtm.Tuple) it.next();
                out.add( t.ctxt(), t.method(), e.stmt(), e.kind(),
                    e.tgtc(), e.tgtm() );
                change = true;
            }
            m2c.add( MethodContext.v( t.method(), t.ctxt() ) );
        }
        return change;
    }
}

